﻿//  This file is part of Just-In WhitePages.
//
//   Just-In WhitePages is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Just-In WhitePages is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Just-In WhitePages.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.DirectoryServices;
using System.Text;
using System.Xml;
using System.Data;
using System.Configuration;

namespace WhitePages
{
    public partial class frmDefault : System.Web.UI.Page
    {
        protected void ProcessFilters()
        {
            string q = Request.QueryString["q"];
            string gn = Request.QueryString["gn"];
            string sn = Request.QueryString["sn"];

            if (!string.IsNullOrEmpty(q))
            {
                txtQuery.Text = Request.QueryString["q"];
                ProcessSimpleFilter();
            }
            else if (!string.IsNullOrEmpty(gn) || !string.IsNullOrEmpty(sn))
            {
                ProcessPrefixFilter();
            }
        }

        protected void ProcessPrefixFilter()
        {
            string firstLastTemplate = "(|(&(sn={0}*)(givenName={1}*))(cn={1}* {0}*))";
            string lastFirstTemplate = "(|(&(sn={0}*)(givenName={1}*))(cn={0}*, {1}*))";
            string [] values = new string[2];
            string filter;
            bool overLimit;
            DataView results;

            values[0] = Request.QueryString["sn"];
            values[1] = Request.QueryString["gn"];

            if (Settings.NamingPattern == NamingPatterns.LastCommaFirst)
                filter = string.Format(lastFirstTemplate, values);
            else
                filter = string.Format(firstLastTemplate, values);

            results = DirectoryLookup.Find(WhitePages.Settings.GridProperties, filter, Settings.Sort, out overLimit);

            LoadResults(results);                                                                                    
        }

        protected void ProcessSimpleFilter()
        {
            bool overLimit;
            string[] parts = txtQuery.Text.Split('=');
            DataView results;

            if (parts.Length == 1)
                results = DirectoryLookup.FindByName(WhitePages.Settings.GridProperties, txtQuery.Text, Settings.Sort, out overLimit);
            else
                results = DirectoryLookup.Find(WhitePages.Settings.GridProperties, txtQuery.Text, Settings.Sort, out overLimit);
            
            LoadResults(results);        
        }

        protected void ProcessAdvancedFilters()
        {
            bool overLimit;
            DataView results;
            StringBuilder filter = new StringBuilder();
            string value;
            bool hasOneFilter = false;

            filter.Append("(&");

            foreach (string p in Settings.AdvancedProperties)
            {
                value = Request.Form["adv:" + p];
                if (value != null && value.Length > 0)
                {
                    if (value.Contains("*"))
                        filter.AppendFormat("({0}={1})", p, value);
                    else
                        filter.AppendFormat("({0}=*{1}*)", p, value);
                    hasOneFilter = true;
                }
            }

            filter.Append(")");

            if (!hasOneFilter)
                return;
                        
            results = DirectoryLookup.Find(WhitePages.Settings.GridProperties, filter.ToString(), Settings.Sort, out overLimit);

            LoadResults(results);        
        }

        protected void LoadResults(DataView results)
        {            
            if (results.Count >  Settings.ResultLimit)
                lblOverLimit.Visible = true;

            if (results.Count <= 0)
            {
                lblNoResults.Visible = true;
                repResults.Visible = false;
            }

            if (results.Count == 1 && Settings.ShowDetailForOneResult)
                Response.Redirect(string.Format("Detail.aspx?dn={0}", Server.UrlEncode((string)results[0]["distinguishedName"])));

            repResults.DataSource = results;
            repResults.DataBind();

            phMessage.Visible = false;
        }

        public string GetHeaders()
        {
            StringBuilder sb = new StringBuilder();
            foreach (string p in Settings.GridProperties)
            { 
                sb.Append("<th>");
                sb.Append(this.GetPropertyName(p));
                sb.Append("</th>");
            }
            return sb.ToString();

        }

        public string GetResultRow(object dataItem)
        {
            DataRowView sr = (DataRowView)dataItem;
            StringBuilder sb = new StringBuilder();
            WhitePages.Directory.PropertyInfo propInfo;

            foreach (string p in Settings.GridProperties)
            {
                propInfo = DirectoryLookup.GetPropertyInfo(p);
                sb.Append("<td>");
                util.EncodeValue(sb, sr[p], propInfo, false);
                sb.Append("</td>");
            }
            return sb.ToString();
        }

        public string GetPropertyName(string propertyName)
        {
           return DirectoryLookup.GetPropertyInfo(propertyName).DisplayText;
        }

        public string GetPrefixUrl(string propName, string value)
        {
            return util.BuildUrl(this.Context, new string[]{"q",propName}, new string[]{"",value});
        }

        public string GetPrefixClass(string propName,string prefix)
        {
            if (Request.QueryString[propName]==prefix)
                return "selected";
            else
                return "";
        }
                
        protected void LoadPrefix()
        {
            List<string> prefixes = new List<string>();
            for (byte b = 65; b <= 90; b+=1)
                prefixes.Add(System.Text.ASCIIEncoding.ASCII.GetString(new byte[]{b}));
            
            repPrefix.DataSource = prefixes;
            repPrefix.DataBind();

            repPrefix2.DataSource = prefixes;
            repPrefix2.DataBind();
        }


        protected void Page_Load(object sender, EventArgs e)
        {
            if (!this.IsPostBack)
            {
                ProcessFilters();
                if (Request.QueryString["adv"] == "1")
                {
                    this.mvViews.ActiveViewIndex = 1;
                }
            }

            if (this.mvViews.ActiveViewIndex == 0)
            {
                this.LoadPrefix();
            }
            else if (this.mvViews.ActiveViewIndex == 1)
            {
                this.dlAdvanced.DataSource = Settings.AdvancedProperties;
                this.dlAdvanced.DataBind();
            }
            this.Title = ConfigurationManager.AppSettings["Title"];


        }

        protected void btnFind_Click(object sender, EventArgs e)
        {
            ProcessSimpleFilter();
        }

        protected void btnFindAdvanced_Click(object sender, EventArgs e)
        {
            ProcessAdvancedFilters();
        }

    }
}
