﻿//  This file is part of Just-In WhitePages.
//
//   Just-In WhitePages is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Just-In WhitePages is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Just-In WhitePages.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Text;
using System.DirectoryServices;
using System.Xml;
using System.Security;
using System.Security.Principal;
using System.Data;
using WhitePages.Directory;
using System.Configuration;


namespace WhitePages
{
    public partial class Detail : System.Web.UI.Page
    {
        private string[] DetailProperties;
        private string[] EditProperties;
        private DirectoryEntry ResultRecord;
        private bool IsEditing = false;
        private bool CanEdit = false;
        private bool CanEditWithoutPassword = false;
        private bool IsAdminUser = false;
        private bool IsSelf = false;
        private bool CanCommitChanges = false;
        private string distinguishedName;

        protected void InitPageProperties()
        {
            this.distinguishedName = Request.QueryString["dn"];
            this.ResultRecord = DirectoryLookup.GetRecord(this.distinguishedName);            
                        
            if (this.User.Identity.IsAuthenticated && this.User is WindowsPrincipal)
            {
                if (this.ResultRecord.Properties.Contains("objectSid"))
                {
                    SecurityIdentifier recordSid = new SecurityIdentifier((byte[])this.ResultRecord.Properties["objectSid"][0], 0);
                    WindowsIdentity winuser = (WindowsIdentity)this.User.Identity;
                    if ((winuser.User.Equals(recordSid) && Settings.SelfEditEnabled))
                    {
                        this.CanEdit = true;
                        this.IsSelf = true;
                    }
                }
                if (util.UserIsInAdminRoles(this.User))
                {
                    this.CanEdit = true;
                    this.CanEditWithoutPassword = true;
                    this.CanCommitChanges = true;
                    this.IsAdminUser = true;
                }
            }

            if (Request.QueryString["edit"] == "1")
            {
                    this.IsEditing = true;
                    this.btnSave.Visible = true;
                    this.phPassword.Visible = !this.CanEditWithoutPassword;
            }

            if (!this.IsEditing && this.CanEdit)
            {
                this.btnEdit.Visible = true;
            }

            if (this.IsEditing && this.IsAdminUser)
                this.EditProperties = Settings.AdminEditProperties;
            else if (this.IsEditing && this.IsSelf)
                this.EditProperties = Settings.SelfEditProperties;
            else if (this.IsEditing)
                throw new Exception("Access Denied");

            if (!this.IsEditing && this.IsAdminUser)
                this.DetailProperties = Settings.AdminProperties;
            else if(!this.IsEditing && this.IsSelf)
                this.DetailProperties = Settings.PrivateProperties;
            else if (!this.IsEditing)
                this.DetailProperties = Settings.PublicProperties;

            this.LoadDirectReports();            
        }

        protected void LoadDirectReports()
        {
            DataView dv = DirectoryLookup.FindAll(string.Format("(manager={0})", this.distinguishedName),"cn",new string[] {WhitePages.Settings.DisplayNameProperty,"title"});
            repDirectReports.DataSource = dv;
            repDirectReports.DataBind();

            repDirectReports.Visible = (dv.Count > 0);
        }

        protected void LoadProperties()
        {
            string [] props;

              if (this.IsEditing)
                    props = this.EditProperties;
                else
                    props = this.DetailProperties;

              DirectoryPropertyAccessor dpa = new DirectoryPropertyAccessor(this.ResultRecord);

              foreach (string p in props)
              {
                  CreatePropertyRow(dpa, p);
              }
        }
        
        protected void CreatePropertyRow(DirectoryPropertyAccessor dpa, string propertyName)
        {
            PropertyInfo propInfo = DirectoryLookup.GetPropertyInfo(propertyName);
            TableRow tr;
            TableCell td;
            TableHeaderCell th;
            TextBox txt;
            HiddenField hf;
            DropDownList ddl;
            LiteralControl lc;
            HyperLink hl;
            RegularExpressionValidator rev;

            string value;
            string hiddenValue = string.Empty;
            DateTime date;
            DirectoryEntry de = null;


            switch (propInfo.Type)
            {
                case "FileTime":
                    date = dpa.GetDate(propertyName);
                    value = (date >= DateTime.MaxValue) ? "(Never)" : date.ToString("g");
                    break;
                case "FileTimeUtc":
                    date = dpa.GetDate(propertyName);
                    value = (date >= DateTime.MaxValue) ? "(Never)" : date.ToString("g");
                    break;
                case "distinguishedName":
                    hiddenValue = dpa.GetString(propertyName);
                    de = dpa.GetDirectoryEntry(propertyName);
                    if (de != null)
                        value = (string)de.Properties[propInfo.DisplayProperty][0];
                    else
                        value = hiddenValue;
                    break;
                default:
                    value = dpa.GetString(propertyName);
                    break;
            }


            tr = new TableRow();                  
            th = new TableHeaderCell();
            th.Text = propInfo.DisplayText;
            tr.Cells.Add(th);

            td = new TableCell();           
            if (this.IsEditing)
            {
                txt = new TextBox();                
                txt.ID = string.Concat("txt_", propertyName);                                               
                switch (propInfo.Editor)
                {
                    case "TextArea":
                        txt.Columns = 40;
                        txt.Rows = 4;
                        txt.TextMode = TextBoxMode.MultiLine;
                        txt.Text = value;
                        td.Controls.Add(txt);
                        break;
                    case "Manager":
                        //todo: fix this to not suck
                        //sb.AppendFormat("<input type='hidden' id='txt_{0}' name='txt_{0}' value='{1}'>", propertyName, hiddenValue);
                        //sb.AppendFormat("<input type='text' disabled id='txt__{0}'", propertyName);
                        //txt.Attributes.Add("disabled", "disabled");
                        //sb.Append("style='width:200px; color: #000;' onkeypress='return false;' onchange='return false;' onkeydown='return false' value='");
                        //sb.Append(value);
                        //sb.Append("'></input>");                        
                        //sb.AppendFormat("<a href='javascript:void()' onclick=\"return OpenFindUser(this,'txt__{0}','txt_{0}');\">Change</a>&nbsp;", propertyName);
                        //sb.AppendFormat("<a href='javascript:void()' onclick=\"return ClearUserField('txt__{0}','txt_{0}');\">Clear</a>", propertyName);                        
                        txt.Text = value;
                        td.Controls.Add(txt);
                        hf = new HiddenField();
                        hf.ID = string.Concat("txt__", propertyName);
                        hf.Value = hiddenValue;
                        td.Controls.Add(hf);

                        hl = new HyperLink();
                        hl.NavigateUrl = "javascript:void()";
                        hl.Attributes.Add("onclick", string.Format("return OpenFindUser(this,'txt_{0}','txt__{0}');", propertyName));
                        hl.Text = "Change";
                        td.Controls.Add(hl);

                        td.Controls.Add(new LiteralControl("&nbsp;"));

                        hl = new HyperLink();
                        hl.NavigateUrl = "javascript:void()";
                        hl.Attributes.Add("onclick",string.Format("return ClearUserField('txt_{0}','txt__{0}');", propertyName));
                        hl.Text = "Clear";
                        td.Controls.Add(hl);


                        break;
                    default:
                        //sb.AppendFormat("<input type='text' id='txt_{0}' name='txt_{0}' ", propertyName);
                        //sb.Append("style='width:200px' value='");
                        //sb.Append(value);
                        //sb.Append("'/>");
                        txt.Text = value;
                        txt.Width = Unit.Pixel(200);
                        td.Controls.Add(txt);
                        break;
                }
                if (propInfo.Options != null && propInfo.Options.Length > 0)
                {
                    ddl = new DropDownList();
                    ddl.ID = string.Concat("ddl__", propertyName);
                    ddl.Attributes.Add("onchange", string.Format("document.getElementById(ClientIdPrefix+'{0}').value=this.value", txt.ID));
                    ddl.Items.Add("");
                    foreach (string s in propInfo.Options)                        
                        ddl.Items.Add(s);                    
                    td.Controls.Add(ddl);
                }
                if (propInfo.Regex.Length > 0)
                {
                    rev = new RegularExpressionValidator();
                    rev.ID = string.Concat("rev_", propertyName);
                    rev.ControlToValidate = string.Concat("txt_", propertyName);
                    rev.ErrorMessage = string.Concat(propInfo.DisplayText, " is not valid");
                    td.Controls.Add(rev);
                }
                tr.Cells.Add(td);
            }
            else
            {
                lc = new LiteralControl();                                
                lc.ID = string.Concat("l_", propertyName);
                if (propInfo.Name == "manager")
                {
                    if (de != null)
                    {
                        hl = new HyperLink();
                        hl.NavigateUrl = string.Format("Detail.aspx?dn={0}", hiddenValue);
                        hl.Text = value;
                        td.Controls.Add(hl);
                        td.Controls.Add(new LiteralControl("&nbsp;:&nbsp;"));
                        hl = new HyperLink();
                        hl.NavigateUrl = string.Format("Chain.aspx?dn={0}", Server.UrlEncode(dpa.GetString("distinguishedName")));
                        hl.Text = "View Chain";
                        td.Controls.Add(hl);
                        //'sb.AppendFormat("&nbsp;:&nbsp;<a href='Chain.aspx?dn={0}'>View Chain</a>", Server.UrlEncode(dpa.GetString("distinguishedName")));
                    }
                    else
                        td.Text = "(none)";
                        //sb.Append("(none)");
                }
                //else if (propInfo.Type == "distinguishedName")
                //{
                //    if (de != null)
                //    {                       
                //        sb.AppendFormat("<a href='Detail.aspx?dn={0}'>{1}</a>", Server.UrlEncode((string)de.Properties["distinguishedName"][0]), value);
                //    }
                //    else
                //        sb.Append("(none)");
                //}
                else
                {
                    td.Text = util.EncodeValue(value, propInfo, true);                    
                }
                tr.Cells.Add(td);
            }
            tblProps.Rows.Add(tr);
        }
        
        
        protected void Page_Init(object sender, EventArgs e)        
        {
            this.InitPageProperties();
            this.LoadProperties();            
        }

        protected void Page_PreLoad(object sender, EventArgs e)
        {
            
        }
        
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!this.IsPostBack)
            {
                this.lblRecordTitle.Text = DirectoryLookup.GetProperty(this.ResultRecord, Settings.TitleProperty);
                txtFindUser.Attributes.Add("onkeypress",string.Format("ClickOnEnter(event,'{0}');",btnFindUser.ClientID));
            }
            if (!this.CanCommitChanges && txtPassword.Text.Length > 0 && DirectoryLookup.AuthenticateUser(this.distinguishedName, txtPassword.Text))
            {
                this.CanCommitChanges = true;
            }
            this.Title = lblRecordTitle.Text;
        }
                           
        protected void btnEdit_Click(object sender, EventArgs e)
        {
            string dn = Request.QueryString["dn"];
            Response.Redirect("Detail.aspx?edit=1&dn=" + dn);
        }

        protected void btnSave_Click(object sender, EventArgs e)
        {
            string value;
            TextBox txt;
            HiddenField hf;

            if (!this.IsValid)
                return;

            if (!this.CanCommitChanges)
            {
                lblError.Text = "You do not have permission to edit this record or the password you entered was incorrect.";
                return;
            }
            
            if (phUserAndPassword.Visible && txtAdminUserName.Text.Length > 0)
            {
                this.ResultRecord = DirectoryLookup.GetRecord(this.distinguishedName, txtAdminUserName.Text, txtAdminPassword.Text);
                try
                {
                    value = this.ResultRecord.Name;
                }
                catch (Exception ex)
                {
                    lblError.Text = ex.Message;
                    return;
                }
            }
            else if (ConfigurationManager.AppSettings["EditUserID"].Length > 0)
            {
                this.ResultRecord = DirectoryLookup.GetRecord(this.distinguishedName, ConfigurationManager.AppSettings["EditUserID"], ConfigurationManager.AppSettings["EditPassword"]);
            }

            DirectoryPropertyAccessor dpa = new DirectoryPropertyAccessor(this.ResultRecord);
            
            foreach (string p in this.EditProperties)
            {
                txt = (TextBox)tblProps.FindControl(string.Concat("txt_", p));
                hf = (HiddenField)tblProps.FindControl(string.Concat("txt__", p));
                if (hf != null)
                    dpa.SetValue(p, hf.Value);
                else if (txt != null)
                    dpa.SetValue(p, txt.Text);
            }
            
            if (!DirectoryLookup.TryCommitChanges(this.ResultRecord))
            {
                if (this.phUserAndPassword.Visible)
                {
                    lblError.Text = "Unable to commit changes to directory entry.";
                }
                else
                {
                    this.phUserAndPassword.Visible = true;
                }
            }
            else
            {
                Response.Redirect("Detail.aspx?dn=" + Server.UrlEncode(this.distinguishedName));
            }
        }

        protected void btnFindUser_Click(object sender, EventArgs e)
        {
            DataView results = DirectoryLookup.FindByName(new string[] { "cn", "displayName" },txtFindUser.Text,"cn");
            repFindUserResults.DataSource = results;
            repFindUserResults.DataBind();
        }

        protected void btnClearFindUser_Click(object sender, EventArgs e)
        {
            txtFindUser.Text = string.Empty;
            repFindUserResults.DataSource = null;
            repFindUserResults.DataBind();
        }

        protected void btnPostBack_Click(object sender, EventArgs e)
        {
          
        }
    }
}
