﻿// -------------------------------------------------------------------------------------------------------------------- 
// <copyright company="{Company}">
//  This file is part of Just-In WhitePages.
//
//   Just-In WhitePages is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Just-In WhitePages is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Just-In WhitePages.  If not, see <http://www.gnu.org/licenses/>.
// </copyright> 
// -------------------------------------------------------------------------------------------------------------------- 
using System;
using System.Collections.Generic;
using System.Web;
using System.Configuration;
using System.Xml;
using WhitePages.Directory;

namespace WhitePages
{
    public enum NamingPatterns
    {
        FirstLast=1,
        LastCommaFirst=2
    }

    public class Settings
    {
        public static string[] GridProperties
        {
            get
            {
                return ConfigurationManager.AppSettings["GridProperties"].Split(',');
            }
        }
        
        public static string[] AdvancedProperties
        {
            get
            {
                return ConfigurationManager.AppSettings["AdvancedProperties"].Split(',');
            }
        }

        public static string[] PublicProperties
        {
            get
            {
                return ConfigurationManager.AppSettings["PublicProperties"].Split(',');
            }
        }

        public static string[] PrivateProperties
        {
            get
            {
                return ConfigurationManager.AppSettings["PrivateProperties"].Split(',');
            }
        }

        public static string[] SelfEditProperties
        {
            get
            {
                return ConfigurationManager.AppSettings["SelfEditProperties"].Split(',');
            }
        }

        public static string[] AdminRoles
        {
            get
            {
                return ConfigurationManager.AppSettings["AdminRoles"].Split(',');
            }
        }
        
        public static string[] AdminProperties
        {
            get
            {
                return ConfigurationManager.AppSettings["AdminProperties"].Split(',');
            }
        }

        public static string[] AdminEditProperties
        {
            get
            {
                return ConfigurationManager.AppSettings["AdminEditProperties"].Split(',');
            }
        }

        public static bool SelfEditEnabled
        {
            get
            {
                return bool.Parse(ConfigurationManager.AppSettings["AllowSelfEdit"]);
            }
        }

        public static bool ShowDetailForOneResult
        {
            get
            {
                return bool.Parse(ConfigurationManager.AppSettings["ShowDetailForOneResult"]);
            }
        }

        public static string TitleProperty
        {
            get
            {
                return ConfigurationManager.AppSettings["TitleProperty"];
            }
        }

        public static string SearchRoot
        {
            get
            {
                return ConfigurationManager.AppSettings["SearchRoot"];
            }
        }

        public static string ServerName
        {
            get
            {
                return ConfigurationManager.AppSettings["ServerName"];
            }
        }

        public static int ResultLimit
        {
            get
            {
                return int.Parse(ConfigurationManager.AppSettings["ResultLimit"]);
            }
        }

        public static int ChainMaxDepth
        {
            get
            {
                return int.Parse(ConfigurationManager.AppSettings["ChainMaxDepth"]);
            }
        }

        public static string Sort
        {
            get
            {
                return ConfigurationManager.AppSettings["Sort"];
            }
        }

        public static string ValidQueryCharacters
        {
            get
            {
                return ConfigurationManager.AppSettings["ValidQueryCharacters"];
            }
        }

        public static NamingPatterns NamingPattern
        {
            get
            {
                return (NamingPatterns)System.Enum.Parse(typeof(NamingPatterns), ConfigurationManager.AppSettings["NamingPattern"]);
            }
        }

        public static string DisplayNameProperty
        {
            get
            {
                return ConfigurationManager.AppSettings["DisplayNameProperty"];
            }
        }

        public static Dictionary<string, PropertyInfo> PropertyNames
        {
            get
            {
                Dictionary<string, PropertyInfo> propertyNames = (Dictionary<string, PropertyInfo>)HttpContext.Current.Application["props"];
                XmlDocument propFile;
                XmlNodeList nodes;
                XmlNodeList optsNodes;
                List<string> optsList;
                if (propertyNames == null)
                {
                    propertyNames = new Dictionary<string, PropertyInfo>(StringComparer.OrdinalIgnoreCase);
                    propFile = new XmlDocument();
                    propFile.Load(HttpContext.Current.Server.MapPath("properties.config"));
                    nodes = propFile.GetElementsByTagName("property");
                    foreach (XmlNode n in nodes)
                    {
                        optsNodes=((XmlElement)n).GetElementsByTagName("option");
                        optsList = new List<string>(optsNodes.Count);
                        if (optsNodes.Count > 0)
                        {                            
                            foreach (XmlNode o in optsNodes)
                                optsList.Add(o.InnerText);
                        }
                        propertyNames.Add(n.Attributes["name"].Value, new PropertyInfo(n.Attributes["name"].Value, n.Attributes["text"].Value, ((XmlElement)n).GetAttribute("editor"), ((XmlElement)n).GetAttribute("type"), ((XmlElement)n).GetAttribute("displayProperty"), ((XmlElement)n).GetAttribute("link"),((XmlElement)n).GetAttribute("regex"),optsList.ToArray()));
                    }
                }

                return propertyNames;
            }
        }

    }
}
