﻿// -------------------------------------------------------------------------------------------------------------------- 
// <copyright company="{Company}">
//  This file is part of Just-In WhitePages.
//
//   Just-In WhitePages is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Just-In WhitePages is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Just-In WhitePages.  If not, see <http://www.gnu.org/licenses/>.
// </copyright> 
// -------------------------------------------------------------------------------------------------------------------- 
using System;
using System.Collections.Generic;
using System.Web;
using System.DirectoryServices;

namespace WhitePages
{
    namespace Directory
    {
    
    public struct PropertyInfo
    {
        public PropertyInfo(string name, string displayText, string editor, string type, string displayProperty, string link,string regex,string[] options)
        {
            this.Name = name;
            if (displayText != null && displayText.Length > 0)
                this.DisplayText = displayText;
            else
                this.DisplayText = name;
            this.Editor = editor;
            this.Type = type;
            this.DisplayProperty = displayProperty;
            this.FormatString = string.Empty;
            this.Link = link;
            this.Regex = regex;
            this.Options = options;
        }
        public string Name;
        public string DisplayText;
        public string Editor;
        public string Type;
        public string DisplayProperty;
        public string FormatString;
        public string Link;
        public string[] Options;
        public string Regex;
    }

    public class DirectoryPropertyAccessor
    {
        public DirectoryPropertyAccessor(DirectoryEntry obj)
        {
            this.DirectoryEntry = obj;
        }

        public DirectoryPropertyAccessor(SearchResult obj)
        {
            this.SearchResult = obj;
        }

        protected DirectoryEntry DirectoryEntry { get; set; }
        protected SearchResult SearchResult { get; set; }

        public object GetValue(string propName, object defaultValue)
        {
            ResultPropertyValueCollection srValues;
            PropertyValueCollection deValues;
            if (this.SearchResult != null)
            {
                srValues = this.SearchResult.Properties[propName];
                if (srValues != null && srValues.Count == 1)
                    return srValues[0];
                else if (srValues != null && srValues.Count > 1)
                {
                    object[] values = new object[srValues.Count];
                    srValues.CopyTo(values, 0);
                    return values;
                }
            }
            if (this.DirectoryEntry != null)
            {
                deValues = this.DirectoryEntry.Properties[propName];
                if (deValues != null && deValues.Count == 1)
                    return deValues[0];
                else if (deValues != null && deValues.Count > 1)
                {
                    object[] values = new object[deValues.Count];
                    deValues.CopyTo(values, 0);
                    return values;
                }
            }

            return defaultValue;
        }

        public void SetValue(string propName, object value)
        {
            if (this.DirectoryEntry == null)
                this.DirectoryEntry = this.SearchResult.GetDirectoryEntry();

            PropertyValueCollection deValues = this.DirectoryEntry.Properties[propName];

            if (value is string && ((string)value).Length <= 0)
                value = null;

            if (deValues.Count <= 0 && value != null)
                deValues.Add(value);
            else if (deValues.Count > 0 && value is object[])
            {
                deValues.Clear();
                deValues.AddRange((object[])value);
            }
            else if (deValues.Count > 0 && value != null)
                deValues[0] = value;
            else if (deValues.Count > 0)
                deValues.Clear();
        }

        public string GetString(string propName)
        {
            return this.GetValue(propName, string.Empty).ToString();
        }

        public string GetString(string propName, string defaultValue)
        {
            return this.GetValue(propName, defaultValue).ToString();
        }

        public long GetInt64(string propName)
        {
            object v = this.GetValue(propName, 0);
            ActiveDs.IADsLargeInteger largeIntValue;

            if (v is long)
                return (long)v;
            if (v is ActiveDs.IADsLargeInteger)
            {
                largeIntValue = (ActiveDs.IADsLargeInteger)v;
                return (long)((uint)largeIntValue.LowPart + (((long)largeIntValue.HighPart) << 32));
            }

            return Convert.ToInt64(v);
        }

        public long GetInt32(string propName)
        {
            object v = this.GetValue(propName, 0);
            if (v is int)
                return (int)v;

            return Convert.ToInt32(v);
        }

        public DateTime GetUtcDate(string propName)
        {
            return GetDate(propName).ToUniversalTime();
        }

        public DateTime GetDate(string propName)
        {
            object v = GetValue(propName, null);
            ActiveDs.IADsLargeInteger largeIntValue;
            long fileTime;

            if (v is long || v is ActiveDs.IADsLargeInteger)
            {
                if (v is long)
                    fileTime = (long)v;
                else
                {
                    largeIntValue = (ActiveDs.IADsLargeInteger)v;
                    fileTime = (long)((uint)largeIntValue.LowPart + (((long)largeIntValue.HighPart) << 32));
                }

                if (fileTime >= 9223372036854775807)
                    return DateTime.MaxValue;
                else
                    return DateTime.FromFileTime(fileTime);
            }                
            return Convert.ToDateTime(v);
        }

        public DirectoryEntry GetDirectoryEntry(string propName)
        {
            string dn = GetString(propName);
            if (dn.Length <= 0)
                return null;

            return DirectoryLookup.OpenDirectoryEntry(dn);
        }

    }

    }
}
