﻿//  This file is part of Arda.Popup
//
//   Arda.Popup is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Arda.Popup is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Arda.Popup.  If not, see <http://www.gnu.org/licenses/>.
Arda = function() {
};

Arda.object = function(e) {
    var el = Arda.getElement(e);
    if (el)
        return el._nativeObject;
};

Arda.setObject = function(e, o) {
    var el = Arda.getElement(e);
    if (el) {
        el._nativeObject = o;
        el._nativeObject.element = el;
    }
};

Arda.isReady = false;
Arda.isTestReady = false;
Arda.readyItems = [];

Arda.domReady = function(f) {
    if (Arda.isReady)
        f.call();
    else {
        Arda.testReady();
        Arda.readyItems.push(f);
    }
};

Arda.testReady = function() {
    if (Arda.isReady) return;
    if (Arda.isTestReady) return;

    Arda.isTestReady = true;

    if (document.addEventListener) {
        document.addEventListener("DOMContentLoaded", function() {
            document.removeEventListener("DOMContentLoaded", arguments.callee, false);
            Arda.fireReadyItems();
        }, false);
    } else if (document.attachEvent) {
        document.attachEvent("onreadystatechange", function() {
            if (document.readyState === "complete") {
                document.detachEvent("onreadystatechange", arguments.callee);
                Arda.fireReadyItems();
            }
        });
    }
};

Arda.fireReadyItems = function() {
    var f;

    Arda.isReady = true;
    for (var i = 0; i < Arda.readyItems.length; i++) {
        f = Arda.readyItems[i];
        f.call();
    }
};

Arda.getElement = function(e) {
    if (typeof (e) == "string")
        return document.getElementById(e);
    else
        return e;
};

Arda.getTarget = function(evt) {
    if (evt.srcElement)
        return evt.srcElement;
    else
        return evt.target;
}

Arda.attachToEvent = function(el, e, h) {
    if (el.addEventListener) {
        el.addEventListener(e, h, false);
    } else if (el.attachEvent) {
        el.attachEvent("on" + e, h);
    }
    else {
        eval('el.on' + e + ' = h;');
    }
};

Arda.getOffset = function(obj) {
    var y = 0;
    var x = 0;

    if (obj.offsetParent) {
        while (obj.offsetParent) {
            x = x + obj.offsetLeft;
            y = y + obj.offsetTop;
            obj = obj.offsetParent;
        }
    }
    else {
        x = obj.offsetLeft;
        y = obj.offsetTop;
    }
    return { x: x, y: y };
};

Arda.isInside = function(container, x, y) {
    var offset = Arda.getOffset(container);
    var w = container.offsetWidth;
    var h = container.offsetHeight;
    var st = document.documentElement.scrollTop;
    var sl = document.documentElement.scrollLeft;
    var xc = 0;
    var yc = 0;

    offset.x -= sl;
    offset.y -= st;

    if (x > (offset.x + 2) && x < (offset.x + w - 2) && y > (offset.y + 2) && y < (offset.y + h - 2))
        return true;
    else
        return false;
};

Arda.getSnapTo = function(obj, snapTo, pp) {
    var x, y, w, h;
    var offset = Arda.getOffset(snapTo);
    var aw = document.documentElement.clientWidth;
    var ah = document.documentElement.clientHeight;
    var st = document.documentElement.scrollTop;
    var sl = document.documentElement.scrollLeft;

    // space between activator top and top of visible space
    var above = offset.y - st;
    // space between activator bottom and bottom of visible space
    var below = (ah - (offset.y + snapTo.offsetHeight)) + st;
    // space between activator right and right of visible space
    var right = (aw - (offset.x + snapTo.offsetWidth)) + sl;
    // space between activator left and left of visible space
    var left = (offset.x) - sl;
    var delta;

    if (!pp)
        pp = 9;

    if (snapTo) {
        w = obj.offsetWidth;
        h = obj.offsetHeight;

        var LeftX = (offset.x - w);
        var RightX = (offset.x + snapTo.offsetWidth);
        var CenterX = ((snapTo.offsetWidth / 2) + offset.x) - (w / 2);
        var AboveY = (offset.y - h);
        var BelowY = (offset.y + snapTo.offsetHeight);
        var MiddleY = ((snapTo.offsetHeight / 2) + offset.y) - (h / 2);

        // left above
        if (pp == 1) {
            x = LeftX + snapTo.offsetWidth;
            y = AboveY;
        }
        // left middle
        else if (pp == 2) {
            x = LeftX;
            y = MiddleY;
        }
        // left below
        else if (pp == 3) {
            x = LeftX + snapTo.offsetWidth;
            y = BelowY;
        }
        // center above
        else if (pp == 4) {
            x = CenterX;
            y = AboveY;
        }
        // center middle
        else if (pp == 5) {
            x = CenterX;
            y = MiddleY;
        }
        // center below
        else if (pp == 6) {
            x = CenterX;
            y = BelowY;
        }
        // right above
        else if (pp == 7) {
            x = RightX - snapTo.offsetWidth;
            y = AboveY;
        }
        // right middle
        else if (pp == 8) {
            x = RightX;
            y = MiddleY;
        }
        // right below
        else if (pp == 9) {
            x = RightX - snapTo.offsetWidth;
            y = BelowY;
        }

        if (x < sl && w < right)
            x = RightX;
        if (x + w > aw && w < left)
            x = LeftX;
        if (y < st && h < below)
            y = BelowY;
        if (y + h > ah && h < above)
            y = AboveY;

        if (y + h > ah + st)
            y = y - ((y + h) - (ah + st));
        if (y < st)
            y = y + (st - y);

        if (x + w > aw + sl)
            x = x - ((x + w) - (aw + sl));
        if (x < sl)
            x = x + (sl - x);

        return { x: x, y: y };
    }

    return null;
};

Arda.isIE = (/msie/i.test(navigator.userAgent) &&
		   !/opera/i.test(navigator.userAgent));

Arda.isIE6 = (Arda.isIE && /msie 6\.0/i.test(navigator.userAgent));

Arda.EmptyIFrame = function() {
    return "javascript:\'<html></html>\'";
}

function Arda_DoPostBack(eventTarget, eventArgument) {
    var theForm = document.forms['aspnetForm'];
    if (!theForm.onsubmit || (theForm.onsubmit() != false)) {
        theForm.__EVENTTARGET.value = eventTarget;
        theForm.__EVENTARGUMENT.value = eventArgument;
        theForm.submit();
    }
};

function Arda_GetOffsetTop(obj) {
    var p;
    var x = 0;
    if (obj.offsetParent) {
        while (obj.offsetParent) {
            x = x + obj.offsetTop;
            obj = obj.offsetParent;
        }
        x = x + obj.offsetTop;
        return x;
    }
    else {
        return obj.offsetTop;
    }
};

function Arda_GetOffsetLeft(obj) {
    var x = 0;
    if (obj.offsetParent) {
        while (obj.offsetParent) {
            x = x + obj.offsetLeft;
            obj = obj.offsetParent;
        }
        return x;
    }
    else {
        return obj.offsetLeft;
    }
};

function Arda_GetXmlHttpRequest() {
    var xmlHttp;
    try {
        xmlHttp = new ActiveXObject("Msxml2.XMLHTTP");
    }
    catch (e) {
        try {
            xmlHttp = new ActiveXObject("Microsoft.XMLHTTP");
        }
        catch (e2) {
            xmlHttp = false;
        }
    }
    // Non-ActiveX browsers
    if (!xmlHttp && typeof XMLHttpRequest != 'undefined') {
        xmlHttp = new XMLHttpRequest();
    }
    return xmlHttp;
};

function Arda_DoClientCallback(request, argument, callback) {
    var url;
    xml_XmlHttpCallback = callback;
    try {
        url = "?__ccb=1&req=" + request + "&arg=" + argument;
        req = xml_GetXmlHttpRequest();
        req.open("GET", url, true);
        req.onreadystatechange = xml_OnServerResponse;
        req.send(null);
        xml_XmlHttpRequest = req;
        return true;
    }
    catch (e) {
        return false;
    }
};

function Arda_OnServerResponse() {
    if (xml_XmlHttpRequest && xml_XmlHttpRequest.readyState == 4) {
        xml_XmlHttpCallback(xml_XmlHttpRequest);
    }
}

function Arda_GetXmlDocument(text) {
    var xmlDoc;
    var parser;
    try {
        xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
        xmlDoc.async = false;
        xmlDoc.loadXML(text);
    }
    catch (e) {
        parser = new DOMParser();
        xmlDoc = parser.parseFromString(text, "text/xml");
    }
    return xmlDoc;
};

function Arda_IsInside(container, x, y) {
    var offsetLeft = Arda_GetOffsetLeft(container, 0);
    var offsetTop = Arda_GetOffsetTop(container, 0);
    var w = container.offsetWidth;
    var h = container.offsetHeight;
    var st = document.documentElement.scrollTop;
    var sl = document.documentElement.scrollLeft;
    var xc = 0;
    var yc = 0;

    offsetLeft -= sl;
    offsetTop -= st;


    if (x > (offsetLeft + 2) && x < (offsetLeft + w - 2) && y > (offsetTop + 2) && y < (offsetTop + h - 2)) {
        return true;
    }
    else
        return false;

};

function Arda_CancelEvent(e) {
    if (typeof e.stopPropegation == "function")
        e.stopPropegation();
    else if (typeof e.cancelBubble != "undefined")
        e.cancelBubble = true;
    if (typeof e.preventDefault == "function")
        e.preventDefault();
};

function Arda_GetEvent(e) {
    if (e == null)
        return window.event;
    else
        return e;
};


function Arda_ToggleElement(eid0, eid1, eid2, eid3) {
    var r;
    if (eid0 != null) r = Arda_ToggleElementInternal(eid0);
    if (eid1 != null) r = Arda_ToggleElementInternal(eid1);
    if (eid2 != null) r = Arda_ToggleElementInternal(eid2);
    if (eid3 != null) r = Arda_ToggleElementInternal(eid3);
    return r;
};

function Arda_ToggleElementInternal(eid) {
    var e = document.getElementById(eid);

    if (e.style.display && e.style.display == 'none') {
        e.style.display = '';
        return true;
    }
    else {
        e.style.display = 'none';
        return false;
    }
};

function Arda_ExpandElement(e, className, height) {
    if (height) {
        e.oldHeight = e.style.height;
        e.style.height = height;
    }
    if (className) {
        e.oldClassName = className;
        e.className = className;
    }
};

function Arda_CollapseElement(e) {
    if (e.oldHeight)
        e.style.height = e.oldHeight;
    if (e.oldClassName)
        e.className = e.oldClassName;
};

