﻿//  This file is part of Arda.Popup
//
//   Arda.Popup is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Arda.Popup is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Arda.Popup.  If not, see <http://www.gnu.org/licenses/>.
Arda.Popup = function(container, settings) {
    this._container = Arda.getElement(container);
    this._settings = settings;
    this._init = false;

    Arda.setObject(container, this);

    this._hideTimer = null;
    this._showTimer = null;
    this._visible = false;

    if (settings && !settings.trackState)
        this._container.style.display = 'none';
    else if (settings && settings.trackState) {
        this._trackState = true;
    }

    this._activator = null;
    this._snapToActivator = true;
    this._prefPos = 9;

    if (settings && typeof (settings.snapToActivator) != 'undefined')
        this._snapToActivator = settings.snapToActivator;

    if (settings && settings.callback)
        this._callback = settings.callback;

    if (settings && settings.prefPos)
        this._prefPos = settings.prefPos;

    if (settings && settings.onShow)
        this._onShow = settings.onShow;
    if (settings && settings.onHide)
        this._onHide = settings.onHide;

    //this._container.onmouseout = function(e) { return Arda.Popup.handleMouseOut(this._popup, e); };
    //this._container.onmouseover = function(e) { return Arda.Popup.handleMouseOver(this._popup, e); };
    //this._container.style.position='absolute';
};

Arda.Popup.create = function(id, settings) {
    var p = new Arda.Popup(id, settings);
    Arda.domReady(function() { p.init() });
    return p;
};

// static methods
Arda.Popup.handleMouseOver = function(e, p) {
    e = e ? e : window.event;
    p.cancelHide();
    if (!p.isVisible())
        p.show();
    return true;
};

Arda.Popup.handleMouseOut = function(e, p) {
    e = e ? e : window.event;
    p.cancelHide();
    p.delayHide(500);
    return true;
};

// methods

Arda.Popup.prototype.init = function() {
    if (this._init) return;
    var pop = this;
    if (this._settings && this._settings.mouseHide)
        Arda.attachToEvent(this._container, 'mouseout', function(e) { Arda.Popup.handleMouseOut(e, pop); });
    Arda.attachToEvent(this._container, 'mouseover', function(e) { Arda.Popup.handleMouseOver(e, pop); });
    this._init = true;
};

Arda.Popup.prototype.trackState = function() {
    if (!this._trackState) return;
    var x = document.getElementById("__" + this._container.id + "_X");
    var y = document.getElementById("__" + this._container.id + "_Y");
    var d = document.getElementById("__" + this._container.id + "_D");

    if (x)
        x.value = this._container.style.left;
    if (y)
        y.value = this._container.style.top;
    if (d)
        d.value = this._container.style.display;
};

Arda.Popup.prototype.setPosition = function(x, y) {
    if (x || y)
        this._container.style.position = 'absolute';

    if (x && typeof (x) == 'number')
        this._container.style.left = x + "px";
    else if (x)
        this.container.style.left = x;

    if (y && typeof (y) == 'number')
        this._container.style.top = y + "px";
    else if (y)
        this.container.style.top = y;

    this.trackState();
}
Arda.Popup.prototype.show = function(activator, options) {
    this.cancelHide();
    this._activator = activator;

    if (options && options.snap && options.snapTo) {
        this.snapTo(options.snapTo, this._prefPos);
    }
    else if (((options && options.snap) || this._snapToActivator) && activator) {
        this.snapTo(activator.element, this._prefPos);
    }

    if (options && (options.x || options.y)) {
        this.setPosition(options.x, options.y);
    }

    if (options && options.content) {
        this.replaceContent(options.content);
    }

    this._container.style.display = 'block';
    this._container.style.visibility = '';

    if (this._callback) {
        this.fireCallback(activator._eventArgument);
    }

    this._visible = true;

    if (this._onShow) {
        this._onShow();
    }

    this.trackState();
};
Arda.Popup.prototype.snapTo = function(e, pp) {
    var el = Arda.getElement(e);

    if (!this.isVisible()) {
        this._container.style.display = 'block';
        this._container.style.visibility = 'hidden';
    }
    var pos = Arda.getSnapTo(this._container, e, pp);

    if (pos) {
        this.setPosition(pos.x, pos.y);
    }
};
Arda.Popup.prototype.hide = function() {
    this.cancelHide();
    this._container.style.display = 'none';
    this._visible = false;

    if (this._onHide) {
        this._onHide();
    }

    this.trackState();
};
Arda.Popup.prototype.delayHide = function(d) {
    if (!d)
        d = 500;
    var p = this;
    this._hideTimer = window.setTimeout(function() { p.hide(); }, d);
};

Arda.Popup.prototype.delayShow = function(d, act, opts) {
    if (!d)
        d = 500;
    var p = this;
    this._showTimer = window.setTimeout(function() { p.show(act, opts); }, d);
};

Arda.Popup.prototype.cancelHide = function() {
    if (this._hideTimer)
        window.clearTimeout(this._hideTimer);
    if (this._showTimer)
        window.clearTimeout(this._showTimer);
}
Arda.Popup.prototype.isVisible = function() {
    return this._visible;
}
Arda.Popup.prototype.replaceContent = function(c) {
    this._container.innerHTML = c;
}
Arda.Popup.prototype.fireCallback = function(arg) {
    if (this._callback) {
        this._callback(this, arg);
    }
}
Arda.Popup.prototype.callbackResult = function(r, ctx) {
    ctx.replaceContent(r);
};
Arda.Popup.prototype.getActivatorId = function() {
    if (this._activator)
        return this._activator.element.id;
    else
        return '';
};
Arda.Popup.prototype.createIFrame = function() {
    if (this._container.tagName != 'iframe' && Arda.isIE6) {
    }
}
Arda.Popup.prototype.getContainer = function() {
    return this._container;
};
Arda.Popup.prototype.doPostBack = function(arg) {
    if (__doPostBack)
        __doPostBack(this._settings.uqid, arg);
};

Arda.PopupAct = function(container, action, type, eventArgument, popup, settings) {
    this._container = Arda.getElement(container);
    this._action = action;
    this._type = type;
    this._eventArgument = eventArgument;
    this._popup = popup;
    this._settings = settings;
    this._action = action;
    this._init = false;

    Arda.setObject(container, this);
}
Arda.PopupAct.create = function(el, action, type, eventArgument, popup, settings) {
    var a = new Arda.PopupAct(el, action, type, eventArgument, popup, settings);
    Arda.domReady(function() { a.init() });
    return a;
};
Arda.PopupAct.handleClick = function(e) {
    if (!e) e = window.event;
    var act = Arda.object(Arda.getTarget(e));
    var pop = act.getPopup();

    if ((act._action == "show" || act._action == "showhide") && (!pop.isVisible() || act.element.id != pop.getActivatorId())) {
        Arda.PopupAct.showPop(pop, act, null);
    }
    else if ((act._action == "showhide" || act._action == "hide") && pop.isVisible())
        pop.hide();
    return false;
}
Arda.PopupAct.handleMouseOver = function(e, act) {
    if (!e) e = window.event;
    //var act = Arda.object(Arda.getTarget(e));
    var pop = act.getPopup();

    pop.cancelHide();

    if ((act._action == "show" || act._action == "showhide") && (!pop.isVisible() || act.element.id != pop.getActivatorId())) {
        Arda.PopupAct.showPop(pop, act, null);
    }
}
Arda.PopupAct.handleMouseOut = function(e, act) {
    if (!e) e = window.event;
    var pop = act.getPopup();

    pop.cancelHide();

    if ((act._action == "hide" || act._action == "showhide") && (pop.isVisible() || pop._showTimer != null))
        pop.delayHide();
}
Arda.PopupAct.showPop = function(pop, act, opts) {
    if (act._settings && act._settings.showDelay && act._settings.showDelay > 0)
        pop.delayShow(act._settings.showDelay, act, opts);
    else
        pop.show(act, opts);

    if (act._settings && act._settings.onShow)
        act._settings.onShow();
};


Arda.PopupAct.prototype.getPopup = function() {
    if (typeof (this._popup) == 'string')
        return this._popup = Arda.object(this._popup);
    else
        return this._popup;
};
Arda.PopupAct.prototype.init = function() {
    if (this._init) return;

    //this._popup = Arda.object(this._popup);

    var act = this;

    if (this._type == 'click') {
        Arda.attachToEvent(this._container, 'click', Arda.PopupAct.handleClick);
    }
    else if (this._type == 'hover') {
        Arda.attachToEvent(this._container, 'mouseover', function(e) { Arda.PopupAct.handleMouseOver(e, act); });
        Arda.attachToEvent(this._container, 'mouseout', function(e) { Arda.PopupAct.handleMouseOut(e, act); });
    }

    this._init = true;
};
