﻿//  This file is part of Just-In WhitePages.
//
//   Just-In WhitePages is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Just-In WhitePages is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Just-In WhitePages.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using System.Web;
using System.DirectoryServices;
using System.Text;
using System.Data;
using WhitePages.Directory;
using System.Collections.Specialized;

class util
{

    public static bool UserIsInAdminRoles(System.Security.Principal.IPrincipal user)
    {
        string[] editRoles = WhitePages.Settings.AdminRoles;
        foreach (string role in editRoles)
        {
            if (user.IsInRole(role))
                return true;
        }
        return false;
    }

    public static string EncodeValue(object s, bool replaceLineBreaks)
    {
        if (s is DBNull || s == null)
            return string.Empty;
        s = HttpUtility.HtmlEncode(s.ToString());
        s = ((string)s).Replace("'", "&#39;");
        if (replaceLineBreaks) s = ((string)s).Replace(Environment.NewLine, "<br/>");
        return (string)s;
    }

    public static string DsEval(object de, string propertyName)
    {
        if (de is DirectoryEntry)
            return DirectoryLookup.GetProperty((DirectoryEntry)de, propertyName);
        else
            return string.Empty;
    }

    public static void EncodeValue(StringBuilder sb, object value, PropertyInfo pi, bool replaceLineBreaks)
    {
        value = EncodeValue(value, replaceLineBreaks);
        if (pi.Link == "search")
            sb.AppendFormat("<a href='default.aspx?q={1}%3D{0}'>{0}</a>", value, pi.Name);
        else if (pi.Link == "email")
            sb.AppendFormat("<a href='mailto:{0}'>{0}</a>", value);
        else
            sb.Append(value);
    }


    public static string EncodeValue(object value, PropertyInfo pi, bool replaceLineBreaks)
    {
        value = EncodeValue(value, replaceLineBreaks);
        if (pi.Link == "search")
            return string.Format("<a href='default.aspx?q={1}%3D{0}'>{0}</a>", value, pi.Name);
        else if (pi.Link == "email")
            return string.Format("<a href='mailto:{0}'>{0}</a>", value);
        else
            return value.ToString();
    }

    public static string ConvertEmptyToNull(string s)
    {
        if (s != null && s.Length == 0)
        {
            return null;
        }

        return s;
    }

    public static string BuildUrl(System.Web.HttpContext context, string queryStringKey, string queryStringValue)
    {
        NameValueCollection nv = new NameValueCollection(context.Request.QueryString);
        nv.Set(queryStringKey, queryStringValue);
        return BuildUrl(context, nv);
    }

    public static string BuildUrl(System.Web.HttpContext context, NameValueCollection queryString)
    {
        return BuildUrl(context, context.Request.Url.ToString(), queryString);
    }

    public static string BuildUrl(HttpContext context, string[] keys, string[] values)
    {
        NameValueCollection nv = new NameValueCollection();
        for (int i = 0; i <= keys.Length - 1; i++)
        {
            nv.Add(keys[i], values[i]);
        }

        return BuildUrl(context, nv);
    }


    public static string BuildUrl(HttpContext context, string baseUrl, string[] keys, string[] values)
    {
        NameValueCollection nv = new NameValueCollection();
        for (int i = 0; i <= keys.Length - 1; i++)
        {
            nv.Add(keys[i], values[i]);
        }
        return BuildUrl(context, baseUrl, nv);
    }

    public static string BuildUrl(HttpContext context, string baseUrl, string key, string value)
    {
        NameValueCollection nv = new NameValueCollection();
        nv.Add(key, value);
        return BuildUrl(context, baseUrl, nv);
    }

    public static string BuildUrl(HttpContext context, string baseUrl, NameValueCollection queryString)
    {
        StringBuilder url = new StringBuilder();
        string current = baseUrl;
        NameValueCollection values = new NameValueCollection(context.Request.QueryString);
        NameValueCollection baseValues = HttpUtility.ParseQueryString(new UriBuilder(baseUrl).Query);

        if (current.IndexOf("?") > 0)
        {
            url.Append(current.Substring(0, current.IndexOf("?")));
        }
        else
        {
            url.Append(current);
        }
        url.Append("?");

        foreach (string name in baseValues)
        {
            values.Set(name, baseValues[name]);
        }
        foreach (string name in queryString)
        {
            values.Set(name, queryString[name]);
        }

        for (int i = 0; i <= values.Count - 1; i++)
        {
            url.AppendFormat("{0}={1}", values.Keys[i], values[i]);
            if (i < (values.Count - 1))
            {
                url.Append("&");
            }
        }

        return url.ToString();
    }
    
}